#!/bin/sh
# ----------------------------------------------------------------------
# t1.demo.sh
#
#	Demonstrate the autovacuum locking issue that occurs when
#	autovacuum finds a considerable amount of free space at the
#	end of a relation, tries to truncate it, but ends up in a
#	loop of being killed by other transactions that try to acquire
#	a conflicting lock once per minute.
# ----------------------------------------------------------------------

DBNAME=test_autovacuum
DBUSER=wieck
DBHOST=localhost
DBPORT=54393

DBSIZE=4000

DBHPU="-h $DBHOST -p $DBPORT -U $DBUSER"


# ----
# (Re)create a clean test database and initialize it as a pgbench DB
# ----
dropdb $DBHPU $DBNAME
createdb $DBHPU $DBNAME || exit 1
echo "Database created - initializing pgbench with scaling factor 10"
pgbench $DBHPU -i -s 10 $DBNAME
echo ""


# ----
# We want to bloat the history table, so increase the size of the
# filler attribute and give it a default value.
# ----
echo "----"
echo "Altering history table for easier bloat"
psql $DBHPU -e $DBNAME <<_EOF_
	alter table pgbench_history alter column filler type text;

	alter table pgbench_history alter column filler set default 'qwertyuiopasdfghjklzxcvbnm poiuytrewqlkjhgfdsamnbvcxz QWERTYUIOPASDFGHJKLZXCVBNM MNBVCXZLKJHGFDSAPOIUYTREWQ';
_EOF_
echo ""


# ----
# Generate 10000 transactions, each creating a history row.
# ----
echo "----"
echo "Running 10000 initial pgbench transactions to generate history data"
pgbench $DBHPU -n -c 5 -t 2000 $DBNAME
echo ""


# ----
# Now bloat the history table by just duplicating the content
# repeatedly. 
# ----
echo "----"
MB=`psql $DBHPU -qtA -c "select pg_table_size('pgbench_history') / 1024 / 1024;" $DBNAME`
echo "bloating history table by duplicating content"
while [ $MB -lt $DBSIZE ] ; do
	psql $DBHPU -c "insert into pgbench_history select * from pgbench_history limit 500000;" $DBNAME
	MB=`psql $DBHPU -qtA -c "select pg_table_size('pgbench_history') / 1024 / 1024;" $DBNAME`
	echo "history is ${MB} MB"
done
echo ""


# ----
# We don't have a primary key on the history table, so we use the mtime
# timestamp to identify rows. We remember the current time.
# ----
sleep 2
TS=`psql $DBHPU -qtA -c "select current_timestamp;" $DBNAME`
sleep 2

# ----
# Create another 50 rows that will be left at the end of the table to
# prevent the first round of vacuuming to truncate the table already. 
# ----
echo "----"
echo "Creating trailing 50 rows"
OBLOCKS=`psql $DBHPU -qtA -c "select pg_table_size('pgbench_history') / 8192;" $DBNAME`
pgbench $DBHPU -n -c 5 -t 10 $DBNAME
echo ""
sleep 2


# ----
# Delete the large number of rows at the beginning of the history
# table and turn that into free space by running vacuum.
# ----
echo "----"
echo "Deleting history rows except for the 50 trailing rows"
psql $DBHPU -e -c "delete from pgbench_history where mtime <= '$TS';" $DBNAME
sleep 10
echo "Running vacuum to free the space"
psql $DBHPU -e -c "vacuum analyze pgbench_history;" $DBNAME

# ----
# Start the pgbench, wait for a while, then delete everything in
# the history so that the inserts happen in the freespace at the
# beginning of the table while we free up all the space at the end.
# Do this until the table stopped growing.
# ----
echo "Starting pgbench in the background"
pgbench $DBHPU -l -n -c 10 -t 10000000 -f t1.pgbench.profile $DBNAME &
BENCHPID=$!
sleep 30

BLOCKS=`psql $DBHPU -qtA -c "select pg_table_size('pgbench_history') / 8192;" $DBNAME`
while [ $BLOCKS -ne $OBLOCKS ] ; do
	echo "history has grown from $OBLOCKS to $BLOCKS blocks - deleteing..."
	psql $DBHPU -e -c "delete from pgbench_history;" $DBNAME
	OBLOCKS=$BLOCKS
	sleep 30
	BLOCKS=`psql $DBHPU -qtA -c "select pg_table_size('pgbench_history') / 8192;" $DBNAME`
done
echo ""


echo "----"
echo "Table history has stopped growing"
echo "Stop deleting of history at $BLOCKS blocks."
echo "pgbench pid=$BENCHPID"
